<!DOCTYPE kpartgui>
<kpartgui name="kblackbox" version="4">
<MenuBar>
  <Menu name="game"><text>&amp;Game</text>
	<Action name="game_giveup"/>
	<Action name="game_done"/>
	<Separator/>
	<Action name="game_resize"/>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
	<Action name="options_size" append="save_merge"/>
	<Action name="options_balls" append="save_merge"/>
	<Action name="options_tutorial" append="save_merge"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
	<Action name="game_quit"/>
	<Action name="game_new"/>
	<Action name="game_giveup"/>
	<Action name="game_done"/>
	<Separator/>
	<Action name="help_contents"/>
</ToolBar>
</kpartgui>
