<!DOCTYPE kpartgui>
<kpartgui name="kjumpingcube" version="2">
<MenuBar>
  <Menu name="game"><text>&amp;Game</text>
    <Action name="game_hint"/>
    <Action name="game_stop"/>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Action name="options_field" append="save_merge"/>
    <Action name="options_skill" append="save_merge"/>
    <Action name="options_change_computer1" append="save_merge"/>
    <Action name="options_change_computer2" append="save_merge"/>
    <Action name="options_change_color1" append="save_merge"/>
    <Action name="options_change_color2" append="save_merge"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
    <Action name="game_new"/>
    <Action name="game_open"/>
    <Action name="game_save"/>
    <Separator/>
    <Action name="game_stop"/>
    <Action name="game_hint"/>
    <Action name="edit_undo"/>
    <Separator/>
    <Action name="help_contents"/>
</ToolBar>
<StatusBar/>
</kpartgui>
