<!DOCTYPE kconfig>
<kconfig name="kclickety">

<Group name="Options">
  <Entry name="enable animations" vtype="bool" defaultValue="true">
  <text>Enable animations</text>
  </Entry>

  <Entry name="block size" type="ranged" vtype="int"
         defaultValue="20" minValue="20" maxValue="100">
  <text>Block size:</text>
  </Entry>

  <Entry name="fade intensity" type="ranged"
         vtype="double" defaultValue="1" minValue="0" maxValue="1">
  <text>Fade strength:</text>
  </Entry>

  <Entry name="fade color" vtype="QColor" defaultValue="#000000">
  <text>Fade color:</text>
  </Entry>

  <Entry name="menubar visible" vtype="bool" defaultValue="true">
  </Entry>
</Group>

</kconfig>
