<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kbounce" version="2">

<MenuBar>
	<Menu name="game"><text>&amp;Game</text>
	</Menu>

	<Menu name="background"><text>&amp;Background</text>
		<Action name="background_select"/>
		<Action name="background_show"/>
	</Menu>
</MenuBar>

<ToolBar noMerge="1" name="mainToolBar"><text>Main Toolbar</text>
       <Action name="game_new"/>
       <Action name="game_pause"/>
</ToolBar>


</kpartgui>
