<!DOCTYPE kpartgui>
<kpartgui name="megami" version="2">
<MenuBar>
	<Menu name="game">
		<text>&amp;Game</text>
		<Action name="game_deal"/>
		<Action name="game_bet"/>
		<Action name="game_hit"/>
		<Action name="game_stand"/>
	</Menu>
	<Menu name="settings">
		<text>&amp;Settings</text>
	</Menu>
</MenuBar>
</kpartgui>
