<!DOCTYPE kpartgui>
<kpartgui name="kshisen" version="5">
<MenuBar>
  <Menu name="game"><text>&amp;Game</text>
    <Action name="game_hint"/>
    <Separator/>
    <Action name="game_restart"/>
    <Action name="game_finish"/>
    <Separator/>
    <Action name="game_solvable"/>
  </Menu>
  <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
    <Action name="options_size"/>
    <Action name="options_speed"/>
    <Action name="options_level"/>
    <Action name="options_gravity"/>
    <Action name="options_disallow"/>
    <Separator/>
    <Action name="options_configure_keybinding"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
  <Action name="game_quit"/>
  <Separator/>
  <Action name="edit_undo"/>
  <Action name="edit_redo"/>
  <Separator/>
  <Action name="game_pause"/>
  <Separator/>
  <Action name="help_contents"/>
</ToolBar>
<StatusBar/>
</kpartgui>
