<!DOCTYPE kpartgui>
<kpartgui name="kenolaba" version="3">
 <MenuBar>
   <Menu name="game"><text>&amp;Game</text>
      <Action name="game_stop"/>
      <Action name="game_back"/>
      <Action name="game_forward"/>
      <Action name="game_hint"/>
      <Separator/>
      <Action name="game_net"/>
   </Menu>
   <Menu name="edit"><text>&amp;Edit</text>
      <Action name="edit_modify"/>
      <Separator/>
      <Action name="edit_save"/>
      <Action name="edit_restore"/>
   </Menu>
   <Menu name="settings"><text>&amp;Settings</text>
      <Menu name="settingLevel" append="save_merge"><text>&amp;Level</text>
         <Action name="level_easy"/>
         <Action name="level_normal"/>
         <Action name="level_hard"/>
         <Action name="level_challange"/>
      </Menu>
      <Menu name="computerPlays" append="save_merge"><text>&amp;Computer Plays</text>
         <Action name="iplay_red"/>
         <Action name="iplay_yellow"/>
         <Action name="iplay_both"/>
         <Action name="iplay_none"/>
      </Menu>
      <Action name="option_moveSlow" append="save_merge"/>
      <Action name="option_renderBalls" append="save_merge"/>
      <Action name="option_showSpy" append="save_merge"/>
   </Menu>
 </MenuBar>

 <Menu name="rmbPopup">
   <Menu name="rmbSettings"><text>&amp;Settings</text>
     <Action name="options_show_menubar"/>
     <Action name="options_show_toolbar"/>
     <Action name="options_show_statusbar"/>
     <Separator/>
     <Action name="options_save_options"/>
   </Menu>
   <Separator/>
   <Action name="game_new"/>
   <Action name="game_back"/>
   <Action name="game_hint"/>
   <Action name="game_stop"/>
 </Menu>	

 <ToolBar noMerge="1" name="mainToolBar"><text>Main Toolbar</text>
   <Action name="game_new"/>
   <Action name="edit_modify"/>
   <Separator/>
   <Action name="game_back"/>
   <Action name="game_hint"/>
   <Action name="game_stop"/>
   <Separator/>
   <Action name="game_net"/>
 </ToolBar>
</kpartgui>
