<!DOCTYPE kpartgui>
<kpartgui name="ksmiletris" version="2">
<MenuBar>
  <Menu name="game"><text>&amp;Game</text>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
     <Action name="settings_pieces" append="save_merge"/>
     <Action name="settings_sounds" append="save_merge"/>
  </Menu>
</MenuBar>

</kpartgui>
