<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kbattleship" version="0.8">
<MenuBar>
  <Menu name="game"><text>&amp;Game</text>
    <Action name="serverconnect"/>
    <Action name="newserver"/>
    <Action name="singleplayer"/>
    <Separator/>
    <Action name="highscore"/>
    <Action name="enemyinfo"/>
    <Separator/>
    <Action name="gamequit"/>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Action name="showgrid" append="show_merge"/>
    <Action name="configsound"/>
  </Menu>
</MenuBar>
</kpartgui>
