<?xml version="1.0" encoding="utf-8"?> 
<!DOCTYPE kpartgui>
<kpartgui name="standard_containers" version="12">
<MenuBar>
  <Menu name="game" noMerge="1"><text>&amp;Game</text>
    <Action name="new_game"/>
    <Action name="open"/>
    <Action name="open_recent"/>
    <Separator/>
    <Action name="save"/>
    <Separator/>
    <Action name="choose_game"/>
    <Action name="restart_game"/>
    <Separator/>
    <ActionList name="game_actions"/>
    <Separator/>
    <Action name="game_exit"/>
  </Menu>
  <Menu name="edit" noMerge="1"><text>&amp;Edit</text>
    <Action name="undo_move"/>
  </Menu>
  <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
    <Action name="game_type"/>
    <Action name="backside"/>
    <Action name="wallpaper"/>
    <Action name="animation"/> 
    <Action name="enable_autodrop"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Action name="new_game"/>
  <Action name="restart_game"/>
  <Action name="undo_move"/>
  <ActionList name="game_actions"/>
  <Action name="help_game"/>
  <Action name="game_exit"/>
</ToolBar>
<StatusBar/>
</kpartgui>
