<!DOCTYPE kpartgui>
<kpartgui name="ksirtet" version="7">

<MenuBar>
  <Menu name="multiplayers"><text>&amp;Mode</text>
	<Action name="mp_single_human"/>
    <Action name="mp_arcade"/>
    <Separator/>
	<Action name="mp_human_vs_human"/>
	<Action name="mp_human_vs_computer"/>
	<Separator/>
	<Action name="mp_more"/>
  </Menu>
</MenuBar>

<Menu name="popup">
  <Action name="options_show_menubar"/>
  <Separator/>
  <Action name="game_new"/>
  <Separator/>
  <Action name="game_pause"/>
  <Action name="game_highscores"/>
  <Action name="game_print"/>
  <Separator/>
  <Action name="game_quit"/>
</Menu>

</kpartgui>
