<!DOCTYPE kpartgui>
<kpartgui name="kmahjongg" version="1">
<MenuBar>
  <Menu name="game"><text>&amp;Game</text>
    <Action name="game_new_numeric" append="new_merge" />
    <Action name="game_open_theme" append="new_merge" />
    <Action name="game_open_tileset" append="new_merge" />
    <Action name="game_open_background" append="new_merge" />
    <Action name="game_open_layout" append="new_merge" />
    <Action name="game_save_theme" append="save_merge" />
    <Action name="game_hint" />
    <Action name="game_shuffle" />
    <Action name="game_demo_mode" />
    <Action name="game_show_matching_tiles" />
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
  </Menu>
  <Menu name="edit"><text>&amp;Edit</text>
    <Action name="edit_board_editor" />
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Action name="game_new" />
  <Action name="game_new_numeric" />
  <Action name="game_load" />
  <Action name="game_save" />
  <Action name="move_undo" />
  <Action name="move_redo" />
  <Action name="game_pause" />
  <Action name="game_hint" />
</ToolBar>
</kpartgui>
